import os
import sys
from PyQt5.QtCore import QTimer
from PyQt5 import QtCore
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QFileDialog, QMainWindow, QApplication, QWidget, QComboBox, QLineEdit, QRadioButton, QSlider, QCheckBox, QStyle, QProgressDialog, QMessageBox, QProgressBar
import threading
import configparser

# Make sure the flatbuffers dir can be located on sys.path so that the generated files can find it.
# TODO: Use pip for flatbuffers if they ever get their act together: https://github.com/google/flatbuffers/issues/4507
sys.path.insert(0, os.path.realpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), "../pylibs")))

from rlbot.gui.presets import AgentPreset, LoadoutPreset
from rlbot.gui.index_manager import IndexManager
from rlbot.gui.design.qt_gui import Ui_MainWindow
from rlbot.gui.gui_agent import GUIAgent
from rlbot.gui.preset_editors import CarCustomisationDialog, AgentCustomisationDialog

from rlbot.utils.class_importer import get_python_root
from rlbot.agents.base_agent import BOT_CONFIG_MODULE_HEADER, BOT_NAME_KEY
from rlbot.setup_manager import SetupManager, DEFAULT_RLBOT_CONFIG_LOCATION

from rlbot.parsing.rlbot_config_parser import create_bot_config_layout, get_num_players, TEAM_CONFIGURATION_HEADER
from rlbot.parsing.agent_config_parser import PARTICIPANT_CONFIGURATION_HEADER, PARTICIPANT_LOADOUT_CONFIG_KEY, LOOKS_CONFIG_KEY
from rlbot.parsing.match_settings_config_parser import *


class RLBotQTGui(QMainWindow, Ui_MainWindow):
    def __init__(self):
        """
        Creates a new QT mainwindow with the GUI
        """
        super().__init__()
        self.setupUi(self)
        self.setWindowIcon(QIcon(os.path.join("images", "RLBot_logo.png")))
        self.overall_config = None
        self.index_manager = IndexManager(10)

        self.agents = []
        self.agent_presets = {}
        self.bot_names_to_agent_dict = {}
        self.loadout_presets = {}
        self.current_bot = None
        self.overall_config_timer = None
        self.setup_manager = None
        self.match_process = None
        self.overall_config = None
        self.overall_config_path = None

        self.car_customisation = CarCustomisationDialog(self)
        self.agent_customisation = AgentCustomisationDialog(self)

        if os.path.isfile(DEFAULT_RLBOT_CONFIG_LOCATION):
            self.load_overall_config(DEFAULT_RLBOT_CONFIG_LOCATION)
        else:
            self.load_overall_config("")
            self.frame_3.setDisabled(True)
            self.match_settings_groupbox.setDisabled(True)
            self.cfg_save_pushbutton.setDisabled(True)

        self.init_match_settings()
        self.update_match_settings()

        self.connect_functions()
        self.update_bot_type_combobox()

    def bot_item_drop_event(self, dropped_listwidget, event):
        """
        Switches the team for the dropped agent to the other team
        :param dropped_listwidget: The listwidget belonging to the new team for the agent
        :param event: The QDropEvent containing the source
        :return:
        """
        dragged_listwidget = event.source()
        if dragged_listwidget is dropped_listwidget:  # drops in the same widget
            return
        self.current_bot.set_team(dropped_listwidget != self.blue_listwidget)
        self.update_teams_listwidgets()

    def fixed_indices(self):
        """
        Agents in the GUI might not have following overall indices,
        thereby a file saved through the GUI would cause other bots to start than the GUI when ran
        :return: CustomConfig instance, copy of the overall config which has the indices sorted out
        """
        config = self.overall_config.copy()

        used_indices = sorted(self.index_manager.numbers)
        not_used_indices = [e for e in range(10) if e not in used_indices]
        order = used_indices + not_used_indices
        header = config[PARTICIPANT_CONFIGURATION_HEADER]
        for name, config_value in header.values.items():
            old_values = list(config_value.value)
            for i in range(10):
                config_value.set_value(old_values[order[i]], index=i)
        return config

    def run_button_pressed(self):
        self.match_process = threading.Thread(target=self.start_match)
        self.match_process.start()

    def start_match(self):
        """
        Starts a match with the current configuration
        :return:
        """
        agent_configs_dict = {}
        loadout_configs_dict = {}
        for agent in self.agents:
            i, agent_config, loadout_config = agent.get_configs()
            agent_configs_dict[i] = agent_config
            loadout_configs_dict[i] = loadout_config
        agent_configs = {}
        loadout_configs = {}
        index = 0
        for i in range(10):
            if i in agent_configs_dict:
                agent_configs[index] = agent_configs_dict[i]
                loadout_configs[index] = loadout_configs_dict[i]
                index += 1
        self.setup_manager = SetupManager()
        self.setup_manager.startup()
        self.setup_manager.load_config(self.overall_config, self.overall_config_path, agent_configs, loadout_configs)
        self.setup_manager.launch_bot_processes()
        self.setup_manager.run()
        self.setup_manager.shut_down()

    def connect_functions(self):
        """
        Connects all events to the functions which should be called
        :return:
        """
        # Lambda is sometimes used to prevent passing the event parameter.
        self.cfg_load_pushbutton.clicked.connect(lambda: self.load_overall_config())
        self.cfg_save_pushbutton.clicked.connect(lambda: self.save_overall_config())

        self.blue_listwidget.itemSelectionChanged.connect(self.load_selected_bot)
        self.orange_listwidget.itemSelectionChanged.connect(self.load_selected_bot)
        self.blue_listwidget.dropEvent = lambda event: self.bot_item_drop_event(self.blue_listwidget, event)
        self.orange_listwidget.dropEvent = lambda event: self.bot_item_drop_event(self.orange_listwidget, event)

        self.blue_name_lineedit.editingFinished.connect(self.team_settings_edit_event)
        self.orange_name_lineedit.editingFinished.connect(self.team_settings_edit_event)
        self.blue_color_spinbox.valueChanged.connect(self.team_settings_edit_event)
        self.orange_color_spinbox.valueChanged.connect(self.team_settings_edit_event)

        self.blue_minus_toolbutton.clicked.connect(lambda e: self.remove_agent(self.current_bot))
        self.orange_minus_toolbutton.clicked.connect(lambda e: self.remove_agent(self.current_bot))
        self.blue_plus_toolbutton.clicked.connect(lambda e: self.add_agent_button(team_index=0))
        self.orange_plus_toolbutton.clicked.connect(lambda e: self.add_agent_button(team_index=1))

        for child in self.bot_config_groupbox.findChildren(QWidget):
            if isinstance(child, QLineEdit):
                child.editingFinished.connect(self.bot_config_edit_event)
            elif isinstance(child, QSlider):
                child.valueChanged.connect(self.bot_config_edit_event)
            elif isinstance(child, QRadioButton):
                child.toggled.connect(self.bot_config_edit_event)
            elif isinstance(child, QComboBox):
                child.currentTextChanged.connect(self.bot_config_edit_event)
        self.loadout_preset_toolbutton.clicked.connect(self.car_customisation.popup)
        self.agent_preset_toolbutton.clicked.connect(self.agent_customisation.popup)

        for child in self.match_settings_groupbox.findChildren(QWidget):
            if isinstance(child, QComboBox):
                child.currentTextChanged.connect(self.match_settings_edit_event)
            elif isinstance(child, QCheckBox):
                child.toggled.connect(self.match_settings_edit_event)

        self.run_button.clicked.connect(self.run_button_pressed)

    def bot_config_edit_event(self, value=None):
        """
        Handles the events called when editing a value regarding the bot configuration
        :param value: the new value to store in the config
        :return:
        """
        sender = self.sender()
        if value is None:
            value = sender.text()
        agent = self.current_bot

        if sender is self.bot_type_combobox:
            self.update_bot_type_combobox()

        elif sender is self.blue_radiobutton and value:  # 'and value' check to make sure that one got selected
            if agent.get_team() != 0:
                agent.set_team(0)
                self.update_teams_listwidgets()
                self.blue_listwidget.setCurrentItem(self.blue_listwidget.findItems(self.validate_name(agent.get_name(), agent), QtCore.Qt.MatchExactly)[0])

        elif sender is self.orange_radiobutton and value:
            if agent.get_team() != 1:
                agent.set_team(1)
                self.update_teams_listwidgets()
                self.orange_listwidget.setCurrentItem(self.orange_listwidget.findItems(self.validate_name(agent.get_name(), agent), QtCore.Qt.MatchExactly)[0])

        elif sender is self.ign_lineedit:
            if agent not in self.agents:
                return
            if not agent.get_team():
                listwidget = self.blue_listwidget
            else:
                listwidget = self.orange_listwidget
            name = self.validate_name(value, agent)
            old_name = self.validate_name(agent.ingame_name, agent)
            listwidget.findItems(old_name, QtCore.Qt.MatchExactly)[0].setText(name)
            del self.bot_names_to_agent_dict[old_name]
            agent.set_name(value)
            self.bot_names_to_agent_dict[name] = agent
        elif sender is self.loadout_preset_combobox:
            if value and self.bot_config_groupbox.isEnabled() and self.current_bot is not None:
                self.current_bot.set_loadout_preset(self.loadout_presets[value])
        elif sender is self.agent_preset_combobox:
            if value and self.bot_config_groupbox.isEnabled() and self.current_bot is not None:
                self.current_bot.set_agent_preset(self.agent_presets[value])
        elif sender is self.bot_level_slider:
            agent.set_bot_skill(value / 100)

    def update_bot_type_combobox(self):
        """
        Handles selecting another bot type in the combobox, hides some frames and shows others depending on the setting
        Also saves the new type if there is a bot selected
        :return:
        """
        if self.bot_type_combobox.currentText() == 'RLBot':
            self.rlbot_frame.setHidden(False)
            self.extra_line.setHidden(False)
            self.psyonix_bot_frame.setHidden(True)
        elif self.bot_type_combobox.currentText() == 'Psyonix':
            self.psyonix_bot_frame.setHidden(False)
            self.rlbot_frame.setHidden(True)
            self.extra_line.setHidden(False)
        elif self.bot_type_combobox.currentText() == 'Human' or self.bot_type_combobox.currentText() == 'Party Member Bot' or self.bot_type_combobox.currentText() == 'Controller Passthrough':
            self.psyonix_bot_frame.setHidden(True)
            self.rlbot_frame.setHidden(True)
            self.extra_line.setHidden(True)
        if self.bot_config_groupbox.isEnabled() and self.current_bot is not None:
            config_type = self.bot_type_combobox.currentText().lower().replace(" ", "_")
            self.current_bot.set_participant_type(config_type)

    def team_settings_edit_event(self, value=None):
        """
        Handles the events when editing a value regarding the team settings
        :param value: the new value to store in the config
        :return:
        """
        sender = self.sender()
        if value is None:
            value = sender.text()

        if sender is self.blue_name_lineedit:
                self.overall_config.set_value(TEAM_CONFIGURATION_HEADER, "Team Blue Name", value)
        elif sender is self.orange_name_lineedit:
            self.overall_config.set_value(TEAM_CONFIGURATION_HEADER, "Team Orange Name", value)
        elif sender is self.blue_color_spinbox:
            self.overall_config.set_value(TEAM_CONFIGURATION_HEADER, "Team Blue Color", value)
        elif sender is self.orange_color_spinbox:
            self.overall_config.set_value(TEAM_CONFIGURATION_HEADER, "Team Orange Color", value)

    def update_team_settings(self):
        """
        Sets all team settings widgets to the value in the overall config
        :return:
        """
        self.blue_name_lineedit.setText(self.overall_config.get(TEAM_CONFIGURATION_HEADER, "Team Blue Name"))
        self.orange_name_lineedit.setText(self.overall_config.get(TEAM_CONFIGURATION_HEADER, "Team Orange Name"))
        self.blue_color_spinbox.setValue(self.overall_config.getint(TEAM_CONFIGURATION_HEADER, "Team Blue Color"))
        self.orange_color_spinbox.setValue(self.overall_config.getint(TEAM_CONFIGURATION_HEADER, "Team Orange Color"))

    def load_overall_config(self, config_path=None):
        """
        Loads the overall config from the config path, or asks for a path if config_path is None
        :param config_path: the path to load the overall_config from, if None a path is requested
        :return:
        """
        if self.overall_config is None:
            self.overall_config = create_bot_config_layout()
        GUIAgent.overall_config = self.overall_config
        if config_path is None:
            config_path = QFileDialog.getOpenFileName(self, "Load Overall Config", "", "Config Files (*.cfg)")[0]
            if not config_path:
                self.statusbar.showMessage("No file selected, not loading config", 5000)
                return
        if config_path is None or not os.path.isfile(config_path):
            return
        paths = os.path.splitext(config_path)
        if paths[-1] != ".cfg":
            self.popup_message("This file is not a config file!", "Invalid File Extension", QMessageBox.Warning)
            return
        raw_parser = configparser.RawConfigParser()
        raw_parser.read(config_path)
        for section in self.overall_config.headers.keys():
            if not raw_parser.has_section(section):
                self.popup_message("This file does not have the sections for an overall config, not loading it!", "Invalic Config File", QMessageBox.Warning)
                return
        for item in (self.frame_3, self.match_settings_groupbox, self.cfg_save_pushbutton):
            item.setEnabled(True)
        self.overall_config_path = config_path
        self.overall_config.parse_file(raw_parser, 10)
        self.load_agents()
        self.update_teams_listwidgets()
        self.cfg_file_path_lineedit.setText(self.overall_config_path)
        self.update_team_settings()
        self.car_customisation.update_presets_widgets()
        self.agent_customisation.update_presets_widgets()

    def save_overall_config(self, time_out=0):
        """
        Schedules a save after given time_out
        :param time_out: The amound of milliseconds it should wait before saving
        :return:
        """
        def save():
            if not os.path.exists(self.overall_config_path):
                return
            with open(self.overall_config_path, "w") as f:
                f.write(str(self.fixed_indices()))
        if self.overall_config_timer is None:
            self.overall_config_timer = QTimer()
            self.overall_config_timer.setSingleShot(True)
            self.overall_config_timer.timeout.connect(save)
        save_path = self.overall_config_path
        if save_path is None or not os.path.exists(save_path):
            save_path = QFileDialog.getSaveFileName(self, "Save Overall Config", "", "Config Files (*.cfg)")[0]
            if not save_path:
                self.statusbar.showMessage("Unable to save a config without location", 5000)
                return
            self.overall_config_path = save_path
        self.overall_config_timer.start(time_out)

    def load_selected_bot(self):
        """
        Loads all the values belonging to the new selected agent into the bot_config_groupbox
        :return:
        """
        # prevent processing from itself (clearing the other one processes this)
        if not self.sender().selectedItems():
            return

        blue = True if self.sender() is self.blue_listwidget else False

        if blue:  # deselect the other listbox
            self.orange_listwidget.clearSelection()
        else:
            self.blue_listwidget.clearSelection()
        item_name = self.sender().selectedItems()[0].text()
        agent = self.bot_names_to_agent_dict[item_name]
        if agent is None:  # something went wrong if agent is None
            return

        self.current_bot = agent

        self.bot_config_groupbox.setEnabled(True)  # Make sure that you can edit the bot
        # enable [-] for right listwidget
        if blue:
            self.blue_minus_toolbutton.setDisabled(False)
            self.orange_minus_toolbutton.setDisabled(True)
        else:
            self.orange_minus_toolbutton.setDisabled(False)
            self.blue_minus_toolbutton.setDisabled(True)

        # load the bot parameters into the edit frame
        agent_type = agent.get_participant_type()

        known_types = ['human', 'psyonix', 'rlbot', 'party_member_bot', 'controller_passthrough']
        assert agent_type in known_types, 'Bot has unknown type: %s' % agent_type

        self.bot_type_combobox.setCurrentIndex(known_types.index(agent_type))
        if blue:
            self.blue_radiobutton.setChecked(True)
        else:
            self.orange_radiobutton.setChecked(True)
        self.ign_lineedit.setText(agent.ingame_name)
        self.loadout_preset_combobox.setCurrentText(agent.get_loadout_preset().get_name())
        self.agent_preset_combobox.setCurrentText(agent.get_agent_preset().get_name())
        self.bot_level_slider.setValue(int(agent.get_bot_skill() * 100))

    def update_teams_listwidgets(self):
        """
        Clears all items from the listwidgets and then adds everything from the self.agents list again to the right team
        :return:
        """
        self.bot_names_to_agent_dict.clear()
        self.blue_listwidget.clear()
        self.orange_listwidget.clear()
        for agent in self.agents:
            name = self.validate_name(agent.ingame_name, agent)
            if not agent.get_team():
                self.blue_listwidget.addItem(name)
            else:
                self.orange_listwidget.addItem(name)

            self.bot_names_to_agent_dict[name] = agent

        self.enable_disable_on_bot_select_deselect()

        # if max bot count reached: disable + button
        if not self.index_manager.has_free_slots():
            self.blue_plus_toolbutton.setDisabled(True)
            self.orange_plus_toolbutton.setDisabled(True)
        else:
            self.blue_plus_toolbutton.setDisabled(False)
            self.orange_plus_toolbutton.setDisabled(False)

    def enable_disable_on_bot_select_deselect(self):
        """
        Disables the botconfig groupbox and minus buttons when no bot is selected
        :return:
        """
        if not self.blue_listwidget.selectedItems() and not self.orange_listwidget.selectedItems():
            self.bot_config_groupbox.setDisabled(True)
            self.blue_minus_toolbutton.setDisabled(True)
            self.orange_minus_toolbutton.setDisabled(True)
        else:
            self.bot_config_groupbox.setDisabled(False)

    def validate_name(self, name, agent):
        """
        Finds the modification of name which is not yet in the list
        :param name: the (new) name for the agent
        :param agent: the agent instance to allow the same name as the previous one if necessary
        :return: the best modification of name not yet in a listwidget
        """
        if name in self.bot_names_to_agent_dict and self.bot_names_to_agent_dict[name] is not agent:
            i = 0
            while True:
                if name + " (" + str(i) + ")" in self.bot_names_to_agent_dict and \
                        self.bot_names_to_agent_dict[name + " (" + str(i) + ")"] is not agent:
                    i += 1
                else:
                    value = name + " (" + str(i) + ")"
                    return value
        else:
            return name

    def add_agent_button(self, team_index: int):
        """
        The method to handle the [+] button press, adds an agent to the team
        :param team_index: the team to set for the new agent, 0 for blue and 1 for orange
        :return:
        """
        agent = self.load_agent()
        if agent is None:
            return
        agent.set_team(team_index)
        self.update_teams_listwidgets()
        if agent.get_team() == 0:
            self.blue_listwidget.setCurrentItem(self.blue_listwidget.findItems(self.validate_name(agent.get_name(), agent), QtCore.Qt.MatchExactly)[0])
        else:
            self.orange_listwidget.setCurrentItem(self.orange_listwidget.findItems(self.validate_name(agent.get_name(), agent), QtCore.Qt.MatchExactly)[0])

    def load_agents(self, config_file=None):
        """
        Loads all agents for this team from the rlbot.cfg
        :param config_file:  A config file that is similar to rlbot.cfg
        """
        if config_file is not None:
            self.overall_config = config_file
        self.agents.clear()
        num_participants = get_num_players(self.overall_config)
        for i in range(num_participants):
            self.load_agent(i)

    def load_agent(self, overall_index: int=None):
        """
        Loads all data for overall_index from the overall config and also loads both presets
        :param overall_index: the index of the targeted agent
        :return agent: an Agent (gui_agent) class with all loaded values
        """
        if not self.index_manager.has_free_slots():
            return None
        if overall_index is None:
            overall_index = self.index_manager.get_new_index()
        else:
            self.index_manager.use_index(overall_index)
        agent = self.add_agent(overall_index=overall_index)

        agent_preset = self.add_agent_preset(agent.get_agent_config_path())
        agent.set_agent_preset(agent_preset)

        loadout_file = self.overall_config.get(PARTICIPANT_CONFIGURATION_HEADER, PARTICIPANT_LOADOUT_CONFIG_KEY, overall_index)
        if loadout_file is None or loadout_file == "None":
            directory = os.path.dirname(os.path.realpath(agent.get_agent_config_path()))
            file_path = agent_preset.config.get(BOT_CONFIG_MODULE_HEADER, LOOKS_CONFIG_KEY)
            loadout_file = os.path.realpath(os.path.join(directory, file_path))
        else:
            directory = get_python_root()
            file_path = loadout_file
            loadout_file = os.path.realpath(os.path.join(directory, file_path))
        loadout_preset = self.add_loadout_preset(loadout_file)
        agent.set_loadout_preset(loadout_preset)
        agent.set_name(agent_preset.config.get(BOT_CONFIG_MODULE_HEADER, BOT_NAME_KEY))
        return agent

    def add_agent(self, overall_index=None, team_index=None):
        """
        Creates the agent using self.agent_class and adds it to the index manager.
        :param overall_index: The index of the bot in the config file if it already exists.
        :param team_index: The index of the team to place the agent in
        :return agent: an Agent (gui_agent) with either given index or a free one, returns None if there is no index given and all indices are occupied
        """
        if overall_index is None:
            if not self.index_manager.has_free_slots():
                return
            overall_index = self.index_manager.get_new_index()
        else:
            self.index_manager.use_index(overall_index)
        agent = GUIAgent(overall_index=overall_index)
        if team_index is not None:
            agent.set_team(team_index)

        self.agents.append(agent)
        self.overall_config.set_value(MATCH_CONFIGURATION_HEADER, PARTICIPANT_COUNT_KEY, len(self.agents))
        return agent

    def remove_agent(self, agent: GUIAgent):
        """
        Removes the given agent.
        :param agent: the agent to remove
        :return:
        """
        self.index_manager.free_index(agent.overall_index)
        self.agents.remove(agent)
        self.update_teams_listwidgets()
        self.overall_config.set_value(MATCH_CONFIGURATION_HEADER, PARTICIPANT_COUNT_KEY, len(self.agents))
        if len(self.agents) == 0:
            return
        if agent.get_team() == 0:
            if self.blue_listwidget.count() != 0:
                self.blue_listwidget.setCurrentRow(self.blue_listwidget.count() - 1)
            else:
                self.orange_listwidget.setCurrentRow(self.orange_listwidget.count() - 1)
        else:
            if self.orange_listwidget.count() != 0:
                self.orange_listwidget.setCurrentRow(self.orange_listwidget.count() - 1)
            else:
                self.blue_listwidget.setCurrentRow(self.blue_listwidget.count() - 1)



    def add_loadout_preset(self, file_path: str):
        """
        Loads a preset using file_path with all values from that path loaded
        :param file_path: the path to load the preset from, if invalid a default preset is returned
        :return preset: the loadout preset created
        """
        if file_path is not None and os.path.isfile(file_path):
            name = os.path.basename(file_path).replace(".cfg", "")
        else:
            name = "new preset"
        if name in self.loadout_presets:
            return self.loadout_presets[name]
        preset = LoadoutPreset(name, file_path)
        self.loadout_presets[preset.get_name()] = preset
        return preset

    def add_agent_preset(self, file_path):
        """
        Loads a preset using file_path with all values from that path loaded
        :param file_path: the path to load the preset from, if invalid a default preset is returned
        :return preset: the agent preset created
        """
        if os.path.isfile(file_path):
            name = os.path.basename(file_path).replace(".cfg", "")
        else:
            name = "new preset"
        if name in self.agent_presets:
            return self.agent_presets[name]
        preset = AgentPreset(name, file_path)
        self.agent_presets[preset.get_name()] = preset
        return preset

    def init_match_settings(self):
        """
        Adds all items to the match settings comboboxes
        :return:
        """
        self.mode_type_combobox.addItems(game_mode_types)
        self.map_type_combobox.addItems(map_types)
        self.match_length_combobox.addItems(match_length_types)
        self.boost_type_combobox.addItems(boost_types)

    def update_match_settings(self):
        """
        Sets all match setting widgets to the values in the overall config
        :return:
        """
        self.mode_type_combobox.setCurrentText(self.overall_config.get(MATCH_CONFIGURATION_HEADER, GAME_MODE))
        self.map_type_combobox.setCurrentText(self.overall_config.get(MATCH_CONFIGURATION_HEADER, GAME_MAP))
        self.skip_replays_checkbox.setChecked(self.overall_config.getboolean(MATCH_CONFIGURATION_HEADER, SKIP_REPLAYS))
        self.instant_start_checkbox.setChecked(self.overall_config.getboolean(MATCH_CONFIGURATION_HEADER, INSTANT_START))
        self.match_length_combobox.setCurrentText(self.overall_config.get(MUTATOR_CONFIGURATION_HEADER, MUTATOR_MATCH_LENGTH))
        self.boost_type_combobox.setCurrentText(self.overall_config.get(MUTATOR_CONFIGURATION_HEADER, MUTATOR_BOOST_AMOUNT))

    def match_settings_edit_event(self, value):
        """
        Handles all edits to match settings and sets the config value to the new value
        :param value: the value to apply to the overall config
        :return:
        """
        sender = self.sender()

        if sender is self.mode_type_combobox:
            self.overall_config.set_value(MATCH_CONFIGURATION_HEADER, GAME_MODE, value)
        elif sender is self.map_type_combobox:
            self.overall_config.set_value(MATCH_CONFIGURATION_HEADER, GAME_MAP, value)
        elif sender is self.skip_replays_checkbox:
            self.overall_config.set_value(MATCH_CONFIGURATION_HEADER, SKIP_REPLAYS, value)
        elif sender is self.instant_start_checkbox:
            self.overall_config.set_value(MATCH_CONFIGURATION_HEADER, INSTANT_START, value)
        elif sender is self.match_length_combobox:
            self.overall_config.set_value(MUTATOR_CONFIGURATION_HEADER, MUTATOR_MATCH_LENGTH, value)
        elif sender is self.boost_type_combobox:
            self.overall_config.set_value(MUTATOR_CONFIGURATION_HEADER, MUTATOR_BOOST_AMOUNT, value)

    def popup_message(self, message: str, title: str, icon=QMessageBox.Warning):
        popup = QMessageBox(self)
        popup.setIcon(icon)
        popup.setWindowTitle(title)
        popup.setText(message)
        popup.setStandardButtons(QMessageBox.Ok)
        popup.exec_()

    @staticmethod
    def main():
        """
        Start the GUI
        :return:
        """
        app = QApplication(sys.argv)
        window = RLBotQTGui()
        window.show()
        app.exec_()
